@@ -1,4 +1,4 @@
-/* $Id: nano.c,v 1.68 2000/10/02 04:21:23 astyanax Exp $ */
+/* $Id: nano.c,v 1.75 2000/11/17 01:37:39 astyanax Exp $ */
 /**************************************************************************
  *   nano.c                                                               *
  *                                                                        *
@@ -29,6 +29,8 @@
 #include <sys/stat.h>
 #include <sys/ioctl.h>
 #include <sys/param.h>
+#include <sys/types.h>
+#include <sys/wait.h>
 #include <errno.h>
 #include <ctype.h>
 #include <locale.h>
@@ -66,6 +68,16 @@ static char *help_text_init = "";
 				/* Initial message, not including shortcuts */
 static struct sigaction act;	/* For all out fun signal handlers */
 
+char *last_search = NULL;	/* Last string we searched for */
+char *last_replace = NULL;	/* Last replacement string */
+int search_last_line;		/* Is this the last search line? */
+
+void keypad_on(int yesno)
+{
+    keypad(edit, yesno);
+    keypad(bottomwin, yesno);
+}
+
 /* What we do when we're all set to exit */
 RETSIGTYPE finish(int sigage)
 {
@@ -74,7 +86,11 @@ RETSIGTYPE finish(int sigage)
 	mvwaddstr(bottomwin, 2, 0, hblank);
     } else
 	mvwaddstr(bottomwin, 0, 0, hblank);
-
+    
+    /* Apparently you REALLY can't get away with not calling keypad()
+       or your window looks awful when it exits.  so we just call it right
+       before we exit, muhaha :-) */
+    keypad_on(TRUE);
     wrefresh(bottomwin);
     endwin();
 
@@ -95,12 +111,12 @@ void die(char *msg, ...)
 
     /* if we can't save we have REAL bad problems,
      * but we might as well TRY. */
-    if(filename[0] == '\0') {
+    if (filename[0] == '\0') {
 	write_file("nano.save", 0);
     } else {
 	char buf[BUFSIZ];
-	strncpy(buf,filename,BUFSIZ);
-	strncat(buf,".save",BUFSIZ - strlen(buf));
+	strncpy(buf, filename, BUFSIZ);
+	strncat(buf, ".save", BUFSIZ - strlen(buf));
 	write_file(buf, 0);
     }
     /* Restore the old term settings */
@@ -122,6 +138,7 @@ void print_view_warning(void)
     statusbar(_("Key illegal in VIEW mode"));
 }
 
+
 /* Initialize global variables - no better way for now */
 void global_init(void)
 {
@@ -321,7 +338,7 @@ void usage(void)
 	   (" -T 		--tabsize=[num]		Set width of a tab to num\n"));
 #ifdef HAVE_REGEX_H
     printf(_
-         (" -R		--regexp		Use regular expressions for search\n"));
+	   (" -R		--regexp		Use regular expressions for search\n"));
 #endif
     printf
 	(_
@@ -347,7 +364,7 @@ void usage(void)
 	(_
 	 (" -r [#cols] 	--fill=[#cols]		Set fill cols to (wrap lines at) #cols\n"));
     printf(_
-	   (" -p	 	--pico			Make bottom 2 lines more Pico-like\n"));
+	   (" -p	 	--pico			Emulate Pico as closely as possible\n"));
     printf(_
 	   (" -s [prog] 	--speller=[prog]	Enable alternate speller\n"));
     printf(_
@@ -384,7 +401,7 @@ void usage(void)
     printf(_
 	   (" -r [#cols] 	Set fill cols to (wrap lines at) #cols\n"));
     printf(_(" -s [prog]  	Enable alternate speller\n"));
-    printf(_(" -p 		Make bottom 2 lines more Pico-like\n"));
+    printf(_(" -p 		Emulate Pico as closely as possible\n"));
     printf(_(" -t 		Auto save on exit, don't prompt\n"));
     printf(_(" -v 		View (read only) mode\n"));
     printf(_(" -w 		Don't wrap long lines\n"));
@@ -399,7 +416,8 @@ void version(void)
 {
     printf(_(" nano version %s by Chris Allegretta (compiled %s, %s)\n"),
 	   VERSION, __TIME__, __DATE__);
-    printf(_(" Email: nano@nano-editor.org	Web: http://www.nano-editor.org\n"));
+    printf(_
+	   (" Email: nano@nano-editor.org	Web: http://www.nano-editor.org\n"));
 }
 
 filestruct *make_new_node(filestruct * prevnode)
@@ -419,7 +437,7 @@ filestruct *make_new_node(filestruct * prevnode)
 }
 
 /* Splice a node into an existing filestruct */
-void splice_node(filestruct *begin, filestruct *new, filestruct *end)
+void splice_node(filestruct * begin, filestruct * new, filestruct * end)
 {
     new->next = end;
     new->prev = begin;
@@ -469,7 +487,7 @@ void do_char(char ch)
 {
     /* magic-line: when a character is inserted on the current magic line,
      * it means we need a new one! */
-    if(filebot == current && current->data[0] == '\0') {
+    if (filebot == current && current->data[0] == '\0') {
 	new_magicline();
 	fix_editbot();
     }
@@ -540,7 +558,7 @@ int do_enter(filestruct * inptr)
      */
     if (current_y == editwinrows - 1) {
 	edit_update(current, CENTER);
-	reset_cursor(); 
+	reset_cursor();
     } else {
 	current_y++;
 	edit_refresh();
@@ -624,7 +642,7 @@ void do_wrap(filestruct * inptr, char input_char)
 	    current_word_start_t = i_tabs;
 
 	    while (!isspace((int) inptr->data[i])
-				&& inptr->data[i]) {
+		   && inptr->data[i]) {
 		i++;
 		i_tabs++;
 		if (inptr->data[i] < 32)
@@ -763,7 +781,8 @@ void do_wrap(filestruct * inptr, char input_char)
 
 	    right = current_x - current_word_start;
 	    i = current_word_start - 1;
-	    if (isspace((int)input_char) && (current_x == current_word_start)) {
+	    if (isspace((int) input_char)
+		&& (current_x == current_word_start)) {
 		current_x = current_word_start;
 
 		null_at(inptr->data, current_word_start);
@@ -877,7 +896,7 @@ void check_wrap(filestruct * inptr, char ch)
 	/* Do not wrap if there are no words on or after wrap point. */
 	int char_found = 0;
 
-	while (isspace((int)inptr->data[i]) && inptr->data[i])
+	while (isspace((int) inptr->data[i]) && inptr->data[i])
 	    i++;
 
 	if (!inptr->data[i])
@@ -954,9 +973,9 @@ int do_backspace(void)
 	    editbot = current;
 
 	    /* Recreate the magic line if we're deleting it AND if the
-		line we're on now is NOT blank.  if it is blank we
-		can just use IT for the magic line.   This is how Pico
-		appears to do it, in any case */
+	       line we're on now is NOT blank.  if it is blank we
+	       can just use IT for the magic line.   This is how Pico
+	       appears to do it, in any case */
 	    if (strcmp(current->data, "")) {
 		new_magicline();
 		fix_editbot();
@@ -1010,8 +1029,7 @@ int do_delete(void)
 
 	/* Please see the comment in do_basckspace if you don't understand
 	   this test */
-	if (current == filebot && strcmp(current->data, ""))
-	{
+	if (current == filebot && strcmp(current->data, "")) {
 	    new_magicline();
 	    fix_editbot();
 	    totsize++;
@@ -1033,74 +1051,278 @@ void wrap_reset(void)
     UNSET(SAMELINEWRAP);
 }
 
-/* Stuff we want to do when we exit the spell program one of its many ways */
-void exit_spell(char *tmpfilename, char *foo)
+#ifndef NANO_SMALL
+
+int do_int_spell_fix(char *word)
 {
-    free(foo);
+    char *prevanswer = NULL, *save_search = NULL, *save_replace = NULL;
+    filestruct *begin, *begin_top;
+    int i = 0, beginx, beginx_top;
 
-    if (remove(tmpfilename) == -1)
-	statusbar(_("Error deleting tempfile, ack!"));
-    display_main_list();
+    /* save where we are */
+    begin = current;
+    beginx = current_x + 1;
+
+    /* save the current search/replace strings */
+    search_init_globals();
+    save_search = mallocstrcpy(save_search, last_search);
+    save_replace = mallocstrcpy(save_replace, last_replace);
+
+    /* set search/replace strings to mis-spelt word */
+    prevanswer = mallocstrcpy(prevanswer, word);
+    last_search = mallocstrcpy(last_search, word);
+    last_replace = mallocstrcpy(last_replace, word);
+
+    /* start from the top of file */
+    begin_top = current = fileage;
+    beginx_top = current_x = -1;
+
+    search_last_line = FALSE;
+
+    /* make sure word is still mis-spelt (i.e. when multi-errors) */
+    if (findnextstr(TRUE, begin_top, beginx_top, prevanswer) != NULL)
+    {
+	/* start from the start of this line again */
+	current = begin_top;
+	current_x = beginx_top;
+
+	search_last_line = FALSE;
+
+	/* allow replace word to be corrected */
+	i = statusq(0, replace_list_2, REPLACE_LIST_2_LEN, last_replace, 
+		_("Edit a replacement"));
+
+	do_replace_loop(prevanswer, begin_top, &beginx_top, TRUE, &i);
+    }
+
+    /* restore the search/replace strings */
+    last_search = mallocstrcpy(last_search, save_search);
+    last_replace = mallocstrcpy(last_replace, save_replace);
+
+    /* restore where we were */
+    current = begin;
+    current_x = beginx - 1;
+
+    edit_update(current, CENTER);
+
+    if (i == -1)
+	return FALSE;
+
+    return TRUE;
 }
+#endif
+
+#ifndef NANO_SMALL
+
+/* Integrated spell checking using 'spell' program */
+int do_int_speller(char *tempfile_name)
+{
+    char *read_buff, *read_buff_ptr, *read_buff_word;
+    long pipe_buff_size;
+    int in_fd[2], tempfile_fd;
+    int spell_status;
+    pid_t pid_spell;
+    ssize_t bytesread;
+
+    /* Create a pipe to spell program */
+
+    if (pipe(in_fd) == -1)
+	return FALSE;
+
+    /* A new process to run spell in */
+
+    if ( (pid_spell = fork()) == 0) {
+
+	/* Child continues, (i.e. future spell process) */
+
+	close(in_fd[0]);
+
+	/* replace the standard in with the tempfile */
+
+	if ( (tempfile_fd = open(tempfile_name, O_RDONLY)) == -1) {
+
+	    close(in_fd[1]);
+	    exit(1);
+	}
+
+	if (dup2(tempfile_fd, STDIN_FILENO) != STDIN_FILENO) {
+
+	    close(tempfile_fd);
+	    close(in_fd[1]);
+	    exit(1);
+	}
+	close(tempfile_fd);
+
+	/* send spell's standard out to the pipe */
+
+	if (dup2(in_fd[1], STDOUT_FILENO) != STDOUT_FILENO) {
+
+	    close(in_fd[1]);
+	    exit(1);
+	}
+	close(in_fd[1]);
+
+	/* Start spell program, we are using the PATH here!?!? */
+	execlp("spell", "spell", NULL);
+
+	/* Should not be reached, if spell is found!!! */
+
+	exit(1);
+    }
+
+    /* Parent continues here */
+
+    close(in_fd[1]);
+
+    /* Child process was not forked successfully */
+
+    if (pid_spell < 0) {
+
+	close(in_fd[0]);
+	return FALSE;
+    }
+
+    /* Get system pipe buffer size */
+
+    if ( (pipe_buff_size = fpathconf(in_fd[0], _PC_PIPE_BUF)) < 1) {
+
+	close(in_fd[0]);
+	return FALSE;
+    }
+
+    read_buff = nmalloc( pipe_buff_size + 1 );
+
+    /* Process the returned spelling errors */
+
+    while ( (bytesread = read(in_fd[0], read_buff, pipe_buff_size)) > 0) {
+
+	read_buff[bytesread] = (char) NULL;
+	read_buff_word = read_buff_ptr = read_buff;
+
+	while (*read_buff_ptr != (char) NULL) {
+
+	    /* Windows version may need to process additional char '\r' */
+
+	    /* Possible problem here if last word not followed by '\n' */
+
+	    if (*read_buff_ptr == '\n') {
+		*read_buff_ptr = (char) NULL;
+		if (!do_int_spell_fix(read_buff_word)) { 
+
+		    close(in_fd[0]);
+		    free(read_buff);
+		    replace_abort();
+
+		    return TRUE;
+	        }
+		read_buff_word = read_buff_ptr;
+		read_buff_word++;
+	    }
+
+	    read_buff_ptr++;
+	}
+    }
+
+    close(in_fd[0]);
+    free(read_buff);
+    replace_abort();
+
+    /* Process end of spell process */
+
+    wait(&spell_status);
+    if (WIFEXITED(spell_status)) {
+	if (WEXITSTATUS(spell_status) != 0)
+	    return FALSE;
+    }
+    else
+	return FALSE;
+
+    return TRUE;
+}
+#endif
+
+#ifndef NANO_SMALL
+
+/* External spell checking */
+int do_alt_speller(char *file_name)
+{
+    int alt_spell_status;
+    pid_t pid_spell;
+
+    endwin();
+
+    /* Start a new process for the alternate speller */
+
+    if ( (pid_spell = fork()) == 0) {
+
+	/* Start alternate spell program, we are using the PATH here!?!? */
+	execlp(alt_speller, alt_speller, file_name, NULL);
+
+	/* Should not be reached, if alternate speller is found!!! */
+
+	exit(1);
+    }
+
+    /* Could not fork?? */
+
+    if (pid_spell < 0)
+	return FALSE;
+
+    /* Wait for alternate speller to complete */
+
+    wait(&alt_spell_status);
+    if (WIFEXITED(alt_spell_status)) {
+	if (WEXITSTATUS(alt_spell_status) != 0)
+	    return FALSE;
+    }
+    else
+	return FALSE;
+
+    refresh();
+    free_filestruct(fileage);
+    global_init();
+    open_file(file_name, 0, 1);
+    edit_update(fileage, CENTER);
+    display_main_list();
+    set_modified();
+
+    return TRUE;
+}
+#endif
 
-/*
- * This is Chris' very ugly spell function.  Someone please make this
- * better =-)
- */
 int do_spell(void)
 {
+
 #ifdef NANO_SMALL
     nano_small_msg();
-    return 1;
+    return (TRUE);
 #else
-    char *temp, *foo;
-    int i, size;
+    char *temp;
+    int spell_res;
 
     if ((temp = tempnam(0, "nano.")) == NULL) {
 	statusbar(_("Could not create a temporary filename: %s"),
-		  strerror(errno));
+			strerror(errno));
 	return 0;
     }
+
     if (write_file(temp, 1) == -1)
 	return 0;
 
-    if (alt_speller) {
-	size = strlen(temp) + strlen(alt_speller) + 2;
-	foo = nmalloc(size);
-	snprintf(foo, size, "%s %s", alt_speller, temp);
-    } else {
+    if (alt_speller)
+	spell_res = do_alt_speller(temp);
+    else
+	spell_res = do_int_speller(temp);
 
-	/* For now, we only try ispell because we're not capable of
-	   handling the normal spell program (yet...) */
-	size = strlen(temp) + 8;
-	foo = nmalloc(size);
-	snprintf(foo, size, "ispell %s", temp);
-    }
+    remove(temp);
 
-    endwin();
-    if (alt_speller) {
-	if ((i = system(foo)) == -1 || i == 32512) {
-	    statusbar(_("Could not invoke spell program \"%s\""),
-		      alt_speller);
-	    exit_spell(temp, foo);
-	    return 0;
-	}
-    } else if ((i = system(foo)) == -1 || i == 32512) {	/* Why 32512? I dont know! */
-	statusbar(_("Could not invoke \"ispell\""));
-	exit_spell(temp, foo);
-	return 0;
-    }
-/*    initscr(); */
-    refresh();
+    if (spell_res)
+	statusbar(_("Finished checking spelling"));
+    else
+	statusbar(_("Spell checking failed"));
+
+    return spell_res;
 
-    free_filestruct(fileage);
-    global_init();
-    open_file(temp, 0, 1);
-    edit_update(fileage, CENTER);
-    set_modified();
-    exit_spell(temp, foo);
-    statusbar(_("Finished checking spelling"));
-    return 1;
 #endif
 }
 
@@ -1185,20 +1407,19 @@ void do_mouse(void)
     }
     current_x = mevent.x;
     placewewant = current_x;
-    while(foo < current_x) {
-	if(current->data[foo] == NANO_CONTROL_I) {
+    while (foo < current_x) {
+	if (current->data[foo] == NANO_CONTROL_I) {
 	    current_x -= tabsize - (foo % tabsize);
 	    tab_found = 1;
-	} else if(current->data[foo] & 0x80)
-	    ;
-	else if(current->data[foo] < 32)
+	} else if (current->data[foo] & 0x80);
+	else if (current->data[foo] < 32)
 	    current_x--;
 	foo++;
     }
     /* This is where tab_found comes in.  I can't figure out why,
      * but without it any line with a tab will place the cursor
      * one character behind.  Whatever, this fixes it. */
-    if(tab_found == 1)
+    if (tab_found == 1)
 	current_x++;
 
     if (current_x > strlen(current->data))
@@ -1227,7 +1448,7 @@ RETSIGTYPE do_suspend(int signal)
     sigaction(SIGTSTP, &act, NULL);
 
     endwin();
-    fprintf(stderr,"\n\n\n\n\nUse \"fg\" to return to nano\n");
+    fprintf(stderr, "\n\n\n\n\nUse \"fg\" to return to nano\n");
     raise(SIGTSTP);
 }
 
@@ -1321,14 +1542,12 @@ void signal_init(void)
 
     if (!ISSET(SUSPEND)) {
 	sigaction(SIGTSTP, &act, NULL);
-    }
-    else
-    {
+    } else {
 	act.sa_handler = do_suspend;
 	sigaction(SIGTSTP, &act, NULL);
 
 	act.sa_handler = do_cont;
-	sigaction (SIGCONT, &act, NULL);
+	sigaction(SIGCONT, &act, NULL);
     }
 
 
@@ -1350,8 +1569,7 @@ void window_init(void)
 
     /* And the other windows */
     topwin = newwin(2, COLS, 0, 0);
-    bottomwin = newwin(3 - no_help(), COLS, LINES - 3 + no_help(), 0);\
-
+    bottomwin = newwin(3 - no_help(), COLS, LINES - 3 + no_help(), 0);
 }
 
 void mouse_init(void)
@@ -1359,17 +1577,14 @@ void mouse_init(void)
 #ifndef NANO_SMALL
 #ifdef NCURSES_MOUSE_VERSION
     if (ISSET(USE_MOUSE)) {
-	keypad(edit, TRUE);
-	keypad(bottomwin, TRUE);
+	keypad_on(TRUE);
 
 	mousemask(BUTTON1_RELEASED, NULL);
 	mouseinterval(50);
 
-    }
-    else {
+    } else {
 	mousemask(0, NULL);
-	keypad(edit, FALSE);
-	keypad(bottomwin, FALSE);
+	keypad_on(FALSE);
     }
 #endif
 #endif
@@ -1500,8 +1715,8 @@ int do_justify(void)
     slen = strlen(current->data);
     totsize += slen;
 
-    if((strlenpt(current->data) > (fill))
-           && !no_spaces(current->data)) {
+    if ((strlenpt(current->data) > (fill))
+	&& !no_spaces(current->data)) {
 	do {
 	    int i = 0;
 	    int len2 = 0;
@@ -1516,11 +1731,12 @@ int do_justify(void)
 		i = slen;
 	    for (; i > 0; i--) {
 		if (isspace((int) current->data[i]) &&
-		    ((strlenpt(current->data) - strlen(current->data +i)) <=
-		     fill)) break;
+		    ((strlenpt(current->data) - strlen(current->data + i))
+		     <= fill))
+		    break;
 	    }
 	    if (!i)
-	        break;
+		break;
 
 	    current->data[i] = '\0';
 
@@ -1542,8 +1758,8 @@ int do_justify(void)
 	    current = tmpline;
 	    slen -= i + 1;
 	    current_y++;
-        } while ((strlenpt(current->data) > (fill))
-		&& !no_spaces(current->data));
+	} while ((strlenpt(current->data) > (fill))
+		 && !no_spaces(current->data));
     }
 
     if (current->next)
@@ -1567,7 +1783,7 @@ int do_justify(void)
     }
 
     edit_refresh();
-    statusbar("Justify Complete");
+    statusbar(_("Justify Complete"));
     return 1;
 #else
     nano_small_msg();
@@ -1610,13 +1826,13 @@ void help_init(void)
 
 	if (main_list[i].misc1 > KEY_F0 && main_list[i].misc1 <= KEY_F(64))
 	    sofar += snprintf(&buf[sofar], BUFSIZ - sofar, "(F%d)	",
-			     main_list[i].misc1 - KEY_F0);
+			      main_list[i].misc1 - KEY_F0);
 	else
 	    sofar += snprintf(&buf[sofar], BUFSIZ - sofar, "	");
 
 	if (main_list[i].altval > 0)
 	    sofar += snprintf(&buf[sofar], BUFSIZ - sofar, "(M-%c)	",
-			     main_list[i].altval - 32);
+			      main_list[i].altval - 32);
 	else
 	    sofar += snprintf(&buf[sofar], BUFSIZ - sofar, "	");
 
@@ -1631,12 +1847,12 @@ void help_init(void)
 
     /* And the toggles... */
     for (i = 0; i <= TOGGLE_LEN - 1; i++) {
-	sofar = snprintf(buf, BUFSIZ, 
-	"M-%c			", toggles[i].val - 32 );
+	sofar = snprintf(buf, BUFSIZ,
+			 "M-%c			", toggles[i].val - 32);
 
 	if (toggles[i].desc != NULL)
-	    snprintf(&buf[sofar], BUFSIZ - sofar, _("%s enable/disable"), 
-		toggles[i].desc);
+	    snprintf(&buf[sofar], BUFSIZ - sofar, _("%s enable/disable"),
+		     toggles[i].desc);
 
 	strcat(help_text, buf);
 	strcat(help_text, "\n");
@@ -1677,13 +1893,13 @@ void do_toggle(int which)
 
     if (!ISSET(toggles[which].flag)) {
 	if (toggles[which].val == TOGGLE_NOHELP_KEY ||
-	   toggles[which].val == TOGGLE_WRAP_KEY)
+	    toggles[which].val == TOGGLE_WRAP_KEY)
 	    statusbar("%s %s", toggles[which].desc, enabled);
 	else
 	    statusbar("%s %s", toggles[which].desc, disabled);
     } else {
 	if (toggles[which].val == TOGGLE_NOHELP_KEY ||
-	   toggles[which].val == TOGGLE_WRAP_KEY)
+	    toggles[which].val == TOGGLE_WRAP_KEY)
 	    statusbar("%s %s", toggles[which].desc, disabled);
 	else
 	    statusbar("%s %s", toggles[which].desc, enabled);
@@ -1711,7 +1927,7 @@ int main(int argc, char *argv[])
     int option_index = 0;
     struct option long_options[] = {
 #ifdef HAVE_REGEX_H
-        {"regexp", 0, 0, 'R'},
+	{"regexp", 0, 0, 'R'},
 #endif
 	{"version", 0, 0, 'V'},
 	{"const", 0, 0, 'c'},
@@ -1792,7 +2008,7 @@ int main(int argc, char *argv[])
 	    SET(USE_MOUSE);
 	    break;
 	case 'p':
-	    SET(PICO_MSGS);
+	    SET(PICO_MODE);
 	    break;
 	case 'r':
 	    fill = atoi(optarg);
@@ -1830,7 +2046,7 @@ int main(int argc, char *argv[])
     argv0 = strrchr(argv[0], '/');
     if ((argv0 && strstr(argv0, "pico"))
 	|| (!argv0 && strstr(argv[0], "pico")))
-	SET(PICO_MSGS);
+	SET(PICO_MODE);
 
     /* See if there's a non-option in argv (first non-option is the
        filename, if +LINE is not given) */
@@ -1887,8 +2103,7 @@ int main(int argc, char *argv[])
 #ifdef PDCURSES
     /* Must have this for the arrow, et al, keys to even work in 
        PDCurses+cygwin under Windows */
-    keypad(edit, TRUE);
-    keypad(bottomwin, TRUE);
+    keypad_on(TRUE);
 #endif
 
 #ifdef DEBUG
@@ -1925,97 +2140,95 @@ int main(int argc, char *argv[])
 	kbinput = wgetch(edit);
 	if (kbinput == 27) {	/* Grab Alt-key stuff first */
 	    switch (kbinput = wgetch(edit)) {
-	    /* Alt-O, suddenly very important ;) */
+		/* Alt-O, suddenly very important ;) */
 	    case 79:
 		kbinput = wgetch(edit);
 		if (kbinput <= 'S' && kbinput >= 'P')
-		     kbinput = KEY_F(kbinput  - 79);
+		    kbinput = KEY_F(kbinput - 79);
 #ifdef DEBUG
 		else {
-		    	fprintf(stderr, _("I got Alt-O-%c! (%d)\n"),
-			kbinput, kbinput);
-			break;
+		    fprintf(stderr, _("I got Alt-O-%c! (%d)\n"),
+			    kbinput, kbinput);
+		    break;
 		}
 #endif
 		break;
 	    case 91:
 
 		switch (kbinput = wgetch(edit)) {
-		case '1': /* Alt-[-1-[0-5,7-9] = F1-F8 in X at least */
+		case '1':	/* Alt-[-1-[0-5,7-9] = F1-F8 in X at least */
 		    kbinput = wgetch(edit);
 		    if (kbinput >= '1' && kbinput <= '5') {
-			    kbinput = KEY_F(kbinput - 48);
-			    wgetch(edit);
-		    }
-		    else if (kbinput >= '7' && kbinput <= '9') {
-			    kbinput = KEY_F(kbinput - 49);
-			    wgetch(edit);
-		    }
-		    else if (kbinput == 126)
-			    kbinput = KEY_HOME;
+			kbinput = KEY_F(kbinput - 48);
+			wgetch(edit);
+		    } else if (kbinput >= '7' && kbinput <= '9') {
+			kbinput = KEY_F(kbinput - 49);
+			wgetch(edit);
+		    } else if (kbinput == 126)
+			kbinput = KEY_HOME;
 
 #ifdef DEBUG
 		    else {
-		    	    fprintf(stderr, _("I got Alt-[-1-%c! (%d)\n"),
-			    kbinput, kbinput);
-			    break;
+			fprintf(stderr, _("I got Alt-[-1-%c! (%d)\n"),
+				kbinput, kbinput);
+			break;
 		    }
 #endif
 
 		    break;
-		case '2': /* Alt-[-2-[0,1,3,4] = F9-F12 in many terms */
+		case '2':	/* Alt-[-2-[0,1,3,4] = F9-F12 in many terms */
 		    kbinput = wgetch(edit);
 		    switch (kbinput) {
-			case '0':
-			    kbinput = KEY_F(9);
-			    wgetch(edit);
-			    break;
-			case '1':
-			    kbinput = KEY_F(10);
-			    wgetch(edit);
-			    break;
-			case '3':
-			    kbinput = KEY_F(11);
-			    wgetch(edit);
-			    break;
-			case '4':
-			    kbinput = KEY_F(12);
-			    wgetch(edit);
-			    break;			    
-			case 126:	/* Hack, make insert key do something 
-					   usefile, like insert file */
-			    do_insertfile();
-			    keyhandled = 1;
-			    break;
+		    case '0':
+			kbinput = KEY_F(9);
+			wgetch(edit);
+			break;
+		    case '1':
+			kbinput = KEY_F(10);
+			wgetch(edit);
+			break;
+		    case '3':
+			kbinput = KEY_F(11);
+			wgetch(edit);
+			break;
+		    case '4':
+			kbinput = KEY_F(12);
+			wgetch(edit);
+			break;
+		    case 126:	/* Hack, make insert key do something 
+				   usefile, like insert file */
+			do_insertfile();
+			keyhandled = 1;
+			break;
 #ifdef DEBUG
-			default:
-		    	    fprintf(stderr, _("I got Alt-[-2-%c! (%d)\n"),
-			    kbinput, kbinput);
-			    break;
+		    default:
+			fprintf(stderr, _("I got Alt-[-2-%c! (%d)\n"),
+				kbinput, kbinput);
+			break;
 #endif
 
 		    }
 		    break;
-		case '3': /* Alt-[-3 = Delete? */
+		case '3':	/* Alt-[-3 = Delete? */
 		    kbinput = NANO_DELETE_KEY;
 		    wgetch(edit);
 		    break;
-		case '4': /* Alt-[-4 = End? */
+		case '4':	/* Alt-[-4 = End? */
 		    kbinput = NANO_END_KEY;
 		    wgetch(edit);
 		    break;
-		case '5': /* Alt-[-5 = Page Up */
+		case '5':	/* Alt-[-5 = Page Up */
 		    kbinput = KEY_PPAGE;
 		    wgetch(edit);
 		    break;
-		case '6': /* Alt-[-6 = Page Down */
+		case '6':	/* Alt-[-6 = Page Down */
 		    kbinput = KEY_NPAGE;
 		    wgetch(edit);
 		    break;
-		case '[': /* Alt-[-[-[A-E], F1-F5 in linux console */
+		case '[':	/* Alt-[-[-[A-E], F1-F5 in linux console */
 		    kbinput = wgetch(edit);
-		     if (kbinput >= 'A' && kbinput <= 'E')
-		 	kbinput = KEY_F(kbinput - 64);
+		    if (kbinput >= 'A' && kbinput <= 'E')
+			kbinput = KEY_F(kbinput - 64);
 		    break;
 		case 'A':
 		    kbinput = KEY_UP;
@@ -2049,17 +2262,17 @@ int main(int argc, char *argv[])
 		for (i = 0; i <= MAIN_LIST_LEN - 1; i++)
 		    if (kbinput == main_list[i].altval ||
 			kbinput == main_list[i].altval - 32) {
-			    kbinput = main_list[i].val;
-			    break;
+			kbinput = main_list[i].val;
+			break;
 		    }
 #ifndef NANO_SMALL
 		/* And for toggle switches */
 		for (i = 0; i <= TOGGLE_LEN - 1 && !keyhandled; i++)
 		    if (kbinput == toggles[i].val ||
 			kbinput == toggles[i].val - 32) {
-			    do_toggle(i);
-			    keyhandled = 1;
-			    break;
+			do_toggle(i);
+			keyhandled = 1;
+			break;
 		    }
 #endif
 #ifdef DEBUG
@@ -2112,11 +2325,11 @@ int main(int argc, char *argv[])
 	    case -1:
 	    case 410:		/* Must ignore this, it gets sent when we resize */
 #ifdef PDCURSES
-	    case 541:	/* ???? */
-	    case 542:	/* Control and alt in Windows *shrug* */
+	    case 541:		/* ???? */
+	    case 542:		/* Control and alt in Windows *shrug* */
 	    case 544:
 #endif
-		
+
 		break;
 	    default:
 #ifdef DEBUG
