@@ -1,4 +1,4 @@
-/* $Id: nano.h,v 1.13 2000/09/06 14:08:37 astyanax Exp $ */
+/* $Id: nano.h,v 1.17 2000/11/17 01:37:39 astyanax Exp $ */
 /**************************************************************************
  *   nano.h                                                               *
  *                                                                        *
@@ -98,7 +98,7 @@ typedef struct toggle {
 #define MARK_ISSET		(1<<3)
 #define CONSTUPDATE		(1<<4)
 #define NO_HELP			(1<<5)
-#define PICO_MSGS		(1<<6)
+#define PICO_MODE		(1<<6)
 #define FOLLOW_SYMLINKS		(1<<7)
 #define SUSPEND			(1<<8)
 #define NO_WRAP			(1<<9)
@@ -240,6 +240,7 @@ know what you're doing */
 #define MAIN_VISIBLE 12
 #define WHEREIS_LIST_LEN 6
 #define REPLACE_LIST_LEN 6
+#define REPLACE_LIST_2_LEN 3
 #define GOTO_LIST_LEN 3
 #define WRITEFILE_LIST_LEN 1
 #define HELP_LIST_LEN 3
