@@ -1,4 +1,4 @@
-/* $Id: search.c,v 1.23 2000/10/01 17:50:29 astyanax Exp $ */
+/* $Id: search.c,v 1.45 2000/11/18 03:05:50 astyanax Exp $ */
 /**************************************************************************
  *   search.c                                                             *
  *                                                                        *
@@ -23,6 +23,7 @@
 #include <string.h>
 #include <unistd.h>
 #include <stdio.h>
+#include <ctype.h>
 #include "config.h"
 #include "proto.h"
 #include "nano.h"
@@ -34,9 +35,6 @@
 #define _(string) (string)
 #endif
 
-static char last_search[132] = "";	/* Last string we searched for */
-static char last_replace[132] = "";	/* Last replacement string */
-
 /* Regular expression helper functions */
 
 #ifdef HAVE_REGEX_H
@@ -53,6 +51,18 @@ void regexp_cleanup()
 }
 #endif
 
+void search_init_globals(void)
+{
+    if (last_search == NULL) {
+	last_search = nmalloc(1);
+	last_search[0] = 0;
+    }
+    if (last_replace == NULL) {
+	last_replace = nmalloc(1);
+	last_replace[0] = 0;
+    }
+}
+
 /* Set up the system variables for a search or replace.  Returns -1 on
    abort, 0 on success, and 1 on rerun calling program 
    Return -2 to run opposite program (searchg -> replace, replace -> search)
@@ -61,54 +71,93 @@ void regexp_cleanup()
 */
 int search_init(int replacing)
 {
-    int i;
-    char buf[BUFSIZ];
+    int i = 0;
+    char *buf;
     char *prompt, *reprompt = "";
- 
-    if (last_search[0]) {
-	snprintf(buf, BUFSIZ, " [%s]", last_search);
-    } else {
-	buf[0] = '\0';
+    static char *backupstring = NULL;
+
+    search_init_globals();
+
+    buf = nmalloc(strlen(last_search) + 5);
+    buf[0] = 0;
+
+
+     /* Okay, fun time.  backupstring is our holder for what is being 
+	returned from the statusq call.  Using answer for this would be tricky.
+	Here, if we're using PICO_MODE, we only want nano to put the
+	old string back up as editable if it's not the same as last_search.
+
+	Otherwise, if we don't already have a backupstring, set it to
+	last_search.  */
+
+    if (ISSET(PICO_MODE)) {
+	if (backupstring == NULL || !strcmp(backupstring, last_search))
+	    backupstring = mallocstrcpy(backupstring, "");
     }
+    else if (backupstring == NULL)
+	backupstring = mallocstrcpy(backupstring, last_search);
+
+    /* If using Pico messages, we do things the old fashioned way... */
+    if (ISSET(PICO_MODE)) {
+	if (last_search[0]) {
+
+	    /* We use COLS / 3 here because we need to see more on the line */
+	    if (strlen(last_search) > COLS / 3) {
+		snprintf(buf, COLS / 3 + 3, " [%s", last_search);
+		sprintf(&buf[COLS / 3 + 2], "...]");
+	    } else
+		sprintf(buf, " [%s]", last_search);
+	} else {
+	    buf[0] = '\0';
+	}
+    }
+    else
+	strcpy(buf, "");
 
     if (ISSET(USE_REGEXP) && ISSET(CASE_SENSITIVE))
-        prompt = _("Case Sensitive Regexp Search%s%s");
+	prompt = _("Case Sensitive Regexp Search%s%s");
     else if (ISSET(USE_REGEXP))
-        prompt = _("Regexp Search%s%s");
-    else 
-    if (ISSET(CASE_SENSITIVE))
-        prompt = _("Case Sensitive Search%s%s");
+	prompt = _("Regexp Search%s%s");
+    else if (ISSET(CASE_SENSITIVE))
+	prompt = _("Case Sensitive Search%s%s");
     else
-        prompt = _("Search%s%s");
+	prompt = _("Search%s%s");
 
     if (replacing)
 	reprompt = _(" (to replace)");
-	        
-    i = statusq(replacing ? replace_list : whereis_list,
-		replacing ? REPLACE_LIST_LEN : WHEREIS_LIST_LEN, "",
-		prompt, reprompt, buf);
+
+    /* This is now one simple call.  It just does a lot */
+    i = statusq(0, replacing ? replace_list : whereis_list,
+	replacing ? REPLACE_LIST_LEN : WHEREIS_LIST_LEN, backupstring,
+	prompt, reprompt, "");
 
     /* Cancel any search, or just return with no previous search */
     if ((i == -1) || (i < 0 && !last_search[0])) {
 	statusbar(_("Search Cancelled"));
 	reset_cursor();
+	free(backupstring);
+	backupstring = NULL;
 	return -1;
     } else if (i == -2) {	/* Same string */
-	strncpy(answer, last_search, 132);
 #ifdef HAVE_REGEX_H
-        if (ISSET(USE_REGEXP))
-            regexp_init(answer);
+	if (ISSET(USE_REGEXP))
+	    regexp_init(answer);
+#else
+	;
 #endif
     } else if (i == 0) {	/* They entered something new */
-	strncpy(last_search, answer, 132);
 #ifdef HAVE_REGEX_H
-        if (ISSET(USE_REGEXP))
-            regexp_init(answer);
+	if (ISSET(USE_REGEXP))
+	    regexp_init(answer);
 #endif
-	/* Blow away last_replace because they entered a new search
-	   string....uh, right? =) */
+	free(backupstring);
+	backupstring = NULL;
 	last_replace[0] = '\0';
     } else if (i == NANO_CASE_KEY) {	/* They want it case sensitive */
+	free(backupstring);
+	backupstring = NULL;
+	backupstring = mallocstrcpy(backupstring, answer);
+
 	if (ISSET(CASE_SENSITIVE))
 	    UNSET(CASE_SENSITIVE);
 	else
@@ -116,19 +165,40 @@ int search_init(int replacing)
 
 	return 1;
     } else if (i == NANO_OTHERSEARCH_KEY) {
+	backupstring = mallocstrcpy(backupstring, answer);
 	return -2;		/* Call the opposite search function */
     } else if (i == NANO_FROMSEARCHTOGOTO_KEY) {
+	free(backupstring);
+	backupstring = NULL;
 	do_gotoline_void();
 	return -3;
     } else {			/* First line key, etc. */
 	do_early_abort();
+	free(backupstring);
+	backupstring = NULL;
 	return -3;
     }
 
     return 0;
 }
 
-filestruct *findnextstr(int quiet, filestruct * begin, char *needle)
+void not_found_msg(char *str)
+{
+    if (strlen(str) <= COLS / 2)
+	statusbar(_("\"%s\" not found"), str);
+    else {
+	char *foo = NULL;
+
+	foo = mallocstrcpy(foo, str);
+	foo[COLS / 2] = 0;
+	statusbar(_("\"%s...\" not found"), foo);
+
+	free(foo);
+    }
+}
+
+filestruct *findnextstr(int quiet, filestruct * begin, int beginx,
+			char *needle)
 {
     filestruct *fileptr;
     char *searchstr, *found = NULL, *tmp;
@@ -136,62 +206,71 @@ filestruct *findnextstr(int quiet, filestruct * begin, char *needle)
 
     fileptr = current;
 
-    searchstr = &current->data[current_x + 1];
-    /* Look for searchstr until EOF */
-    while (fileptr != NULL &&
-	   (found = strstrwrapper(searchstr, needle)) == NULL) {
+    current_x++;
+
+    /* Are we searching the last line? (i.e. the line where search started) */
+    if ((fileptr == begin) && (current_x < beginx))
+	search_last_line = 1;
+
+    /* Make sure we haven't passed the end of the string */
+    if (strlen(fileptr->data) < current_x)
+	current_x--;
+
+    searchstr = &fileptr->data[current_x];
+
+    /* Look for needle in searchstr */
+    while ((found = strstrwrapper(searchstr, needle)) == NULL) {
+
+	/* finished processing file, get out */
+	if (search_last_line) {
+	    if (!quiet)
+		not_found_msg(needle);
+	    return NULL;
+	}
 
 	fileptr = fileptr->next;
 
 	if (!past_editbot && (fileptr == editbot))
 	    past_editbot = 1;
 
-	if (fileptr == begin)
-	    return NULL;
+	/* EOF reached, wrap around once */
+	if (fileptr == NULL) {
+	    fileptr = fileage;
 
-	if (fileptr != NULL)
-	    searchstr = fileptr->data;
-    }
-
-    /* If we're not at EOF, we found an instance */
-    if (fileptr != NULL) {
-	current = fileptr;
-	current_x = 0;
-	for (tmp = fileptr->data; tmp != found; tmp++)
-	    current_x++;
-
-	if (past_editbot)
-	    edit_update(current, CENTER);
-	placewewant = xplustabs();
-	reset_cursor();
-    } else {			/* We're at EOF, go back to the top, once */
-
-	fileptr = fileage;
-
-	while (fileptr != begin->next &&
-	       (found = strstrwrapper(fileptr->data, needle)) == NULL)
-	    fileptr = fileptr->next;
-
-	if (fileptr == begin->next) {
-	    if (!quiet)
-		statusbar(_("\"%s\" not found"), needle);
-
-	    return NULL;
-	}
-	else {	/* We found something */
-	    current = fileptr;
-	    current_x = 0;
-	    for (tmp = fileptr->data; tmp != found; tmp++)
-		current_x++;
-
-	    edit_update(current, CENTER);
-	    reset_cursor();
+	    past_editbot = 1;
 
 	    if (!quiet)
 		statusbar(_("Search Wrapped"));
-	} 
+	}
+
+	/* Original start line reached */
+	if (fileptr == begin)
+	    search_last_line = 1;
+
+	searchstr = fileptr->data;
     }
 
+    /* We found an instance */
+    current = fileptr;
+    current_x = 0;
+    for (tmp = fileptr->data; tmp != found; tmp++)
+	current_x++;
+
+    /* Ensure we haven't wrap around again! */
+    if ((search_last_line) && (current_x >= beginx)) {
+	if (!quiet)
+	    not_found_msg(needle);
+	return NULL;
+    }
+
+    if (past_editbot)
+	edit_update(fileptr, CENTER);
+    else
+	update_line(current, current_x);
+
+    placewewant = xplustabs();
+    reset_cursor();
+
     return fileptr;
 }
 
@@ -200,12 +279,12 @@ void search_abort(void)
     UNSET(KEEP_CUTBUFFER);
     display_main_list();
     wrefresh(bottomwin);
-    if (ISSET(MARK_ISSET)) 
+    if (ISSET(MARK_ISSET))
 	edit_refresh_clearok();
 
 #ifdef HAVE_REGEX_H
     if (ISSET(REGEXP_COMPILED))
-        regexp_cleanup();
+	regexp_cleanup();
 #endif
 }
 
@@ -216,22 +295,41 @@ int do_search(void)
     filestruct *fileptr = current;
 
     wrap_reset();
-    if ((i = search_init(0)) == -1) {
+    i = search_init(0);
+    switch (i) {
+    case -1:
 	current = fileptr;
 	search_abort();
 	return 0;
-    } else if (i == -3) {
+    case -3:
 	search_abort();
 	return 0;
-    } else if (i == -2) {
+    case -2:
 	do_replace();
 	return 0;
-    } else if (i == 1) {
+    case 1:
 	do_search();
 	search_abort();
 	return 1;
     }
-    findnextstr(0, current, answer);
+
+    /* The sneaky user deleted the previous search string */
+    if (!ISSET(PICO_MODE) && !strcmp(answer, "")) {
+	statusbar(_("Search Cancelled"));
+	search_abort();
+	return 0;
+    }
+
+     /* If answer is now == "", then PICO_MODE is set.  So, copy
+	last_search into answer... */
+
+    if (!strcmp(answer, ""))
+	answer = mallocstrcpy(answer, last_search);
+    else
+	last_search = mallocstrcpy(last_search, answer);
+
+    search_last_line = 0;
+    findnextstr(0, current, current_x, answer);
     search_abort();
     return 1;
 }
@@ -250,6 +348,7 @@ void replace_abort(void)
        does something different later, we can change it back.  For now
        it's just a waste to duplicat code */
     search_abort();
+    placewewant = xplustabs();
 }
 
 #ifdef HAVE_REGEX_H
@@ -261,8 +360,7 @@ int replace_regexp(char *string, int create_flag)
 
     char *c;
     int new_size = strlen(current->data) + 1;
-    int search_match_count = regmatches[0].rm_eo -
-        regmatches[0].rm_so;
+    int search_match_count = regmatches[0].rm_eo - regmatches[0].rm_so;
 
     new_size -= search_match_count;
 
@@ -271,51 +369,51 @@ int replace_regexp(char *string, int create_flag)
 
     c = last_replace;
     while (*c) {
-        if (*c != '\\') {
-            if (create_flag)
-                *string++=*c;
-            c++;
-            new_size++;
-        } else {
-            int num = (int)*(c+1) - (int)'0';
-            if (num >= 1 && num <= 9) {
+	if (*c != '\\') {
+	    if (create_flag)
+		*string++ = *c;
+	    c++;
+	    new_size++;
+	} else {
+	    int num = (int) *(c + 1) - (int) '0';
+	    if (num >= 1 && num <= 9) {
 
-                int i = regmatches[num].rm_so;
+		int i = regmatches[num].rm_so;
 
-                if (num > search_regexp.re_nsub) {
-                    /* Ugh, they specified a subexpression that doesn't
-                       exist.  */
-                    return -1;
-                }
+		if (num > search_regexp.re_nsub) {
+		    /* Ugh, they specified a subexpression that doesn't
+		       exist.  */
+		    return -1;
+		}
 
-                /* Skip over the replacement expression */
-                c+=2;
+		/* Skip over the replacement expression */
+		c += 2;
 
-                /* But add the length of the subexpression to new_size */
-                new_size += regmatches[num].rm_eo - regmatches[num].rm_so;
+		/* But add the length of the subexpression to new_size */
+		new_size += regmatches[num].rm_eo - regmatches[num].rm_so;
 
-                /* And if create_flag is set, append the result of the
-                 * subexpression match to the new line */
-                while (create_flag && i < regmatches[num].rm_eo )
-                    *string++=*(current->data + i++);
-               
-            } else {
-                if (create_flag)
-                    *string++=*c;
-                c++;
-                new_size++;
-            }
-        }
+		/* And if create_flag is set, append the result of the
+		 * subexpression match to the new line */
+		while (create_flag && i < regmatches[num].rm_eo)
+		    *string++ = *(current->data + i++);
+
+	    } else {
+		if (create_flag)
+		    *string++ = *c;
+		c++;
+		new_size++;
+	    }
+	}
     }
 
     if (create_flag)
-        *string = 0;
+	*string = 0;
 
     return new_size;
 }
 #endif
-    
-char *replace_line()
+
+char *replace_line(void)
 {
     char *copy, *tmp;
     int new_line_size;
@@ -324,22 +422,21 @@ char *replace_line()
     /* Calculate size of new line */
 #ifdef HAVE_REGEX_H
     if (ISSET(USE_REGEXP)) {
-        search_match_count = regmatches[0].rm_eo -
-            regmatches[0].rm_so;
-        new_line_size = replace_regexp(NULL, 0);
-        /* If they specified an invalid subexpression in the replace
-         * text, return NULL indicating an error */
-        if (new_line_size < 0)
-            return NULL;
+	search_match_count = regmatches[0].rm_eo - regmatches[0].rm_so;
+	new_line_size = replace_regexp(NULL, 0);
+	/* If they specified an invalid subexpression in the replace
+	 * text, return NULL indicating an error */
+	if (new_line_size < 0)
+	    return NULL;
     } else {
 #else
     {
 #endif
-        search_match_count = strlen(last_search);
-        new_line_size = strlen(current->data) - strlen(last_search) +
-            strlen(last_replace) + 1;
+	search_match_count = strlen(last_search);
+	new_line_size = strlen(current->data) - strlen(last_search) +
+	    strlen(last_replace) + 1;
     }
-    
+
     /* Create buffer */
     copy = nmalloc(new_line_size);
 
@@ -349,10 +446,10 @@ char *replace_line()
 
     /* Replacement Text */
     if (!ISSET(USE_REGEXP))
-        strcat(copy, last_replace);
+	strcat(copy, last_replace);
 #ifdef HAVE_REGEX_H
     else
-        (void)replace_regexp(copy + current_x, 1);
+	(void) replace_regexp(copy + current_x, 1);
 #endif
 
     /* The tail of the original line */
@@ -368,109 +465,72 @@ char *replace_line()
     return copy;
 }
 
-/* Replace a string */
-int do_replace(void)
+int do_replace_loop(char *prevanswer, filestruct *begin, int *beginx,
+			int wholewords, int *i)
 {
-    int i, replaceall = 0, numreplaced = 0, beginx;
-    filestruct *fileptr, *begin;
-    char *copy, prevanswer[132] = "";
+    int replaceall = 0, numreplaced = 0;
+    filestruct *fileptr;
+    char *copy;
 
-    if ((i = search_init(1)) == -1) {
+    switch (*i) {
+    case -1:				/* Aborted enter */
+	if (strcmp(last_replace, ""))
+	    answer = mallocstrcpy(answer, last_replace);
 	statusbar(_("Replace Cancelled"));
 	replace_abort();
 	return 0;
-    } else if (i == 1) {
-	do_replace();
-	return 1;
-    } else if (i == -2) {
-	do_search();
-	return 0;
-    } else if (i == -3) {
-	replace_abort();
-	return 0;
-    }
-    strncpy(prevanswer, answer, 132);
-
-    if (strcmp(last_replace, "")) {	/* There's a previous replace str */
-	i = statusq(replace_list, REPLACE_LIST_LEN, "",
-		    _("Replace with [%s]"), last_replace);
-
-	if (i == -1) {		/* Aborted enter */
-	    strncpy(answer, last_replace, 132);
-	    statusbar(_("Replace Cancelled"));
-	    replace_abort();
-	    return 0;
-	} else if (i == 0)	/* They actually entered something */
-	    strncpy(last_replace, answer, 132);
-	else if (i == NANO_CASE_KEY) {	/* They asked for case sensitivity */
-	    if (ISSET(CASE_SENSITIVE))
-		UNSET(CASE_SENSITIVE);
-	    else
-		SET(CASE_SENSITIVE);
-
-	    do_replace();
-	    return 0;
-	} else if (i != -2 ) {		/* First page, last page, for example could get here */
-
+    case 0:		/* They actually entered something */
+	break;
+    default:
+        if (*i != -2) {	/* First page, last page, for example 
+				   could get here */
 	    do_early_abort();
 	    replace_abort();
 	    return 0;
-	}
-    } else {			/* last_search is empty */
-
-	i = statusq(replace_list, REPLACE_LIST_LEN, "", _("Replace with"));
-	if (i == -1) {
-	    statusbar(_("Replace Cancelled"));
-	    reset_cursor();
-	    replace_abort();
-	    return 0;
-	} else if (i == 0)	/* They entered something new */
-	    strncpy(last_replace, answer, 132);
-	else if (i == NANO_CASE_KEY) {	/* They want it case sensitive */
-	    if (ISSET(CASE_SENSITIVE))
-		UNSET(CASE_SENSITIVE);
-	    else
-		SET(CASE_SENSITIVE);
-
-	    do_replace();
-	    return 1;
-	} else {		/* First line key, etc. */
-
-	    do_early_abort();
-	    replace_abort();
-	    return 0;
-	}
+        }
     }
 
-    /* save where we are */
-    begin = current;
-    beginx = current_x;
-
+    last_replace = mallocstrcpy(last_replace, answer);
     while (1) {
 
-	if (replaceall)
-	    fileptr = findnextstr(1, begin, prevanswer);
-	else
-	    fileptr = findnextstr(0, begin, prevanswer);
+	/* Sweet optimization by Rocco here */
+	fileptr = findnextstr(replaceall, begin, *beginx, prevanswer);
 
 	/* No more matches.  Done! */
 	if (!fileptr)
 	    break;
 
+	/* Make sure only wholewords are found */
+	if (wholewords)
+	{
+	    /* start of line or previous character not a letter */
+	    if ((current_x == 0) || (!isalpha((int) fileptr->data[current_x-1])))
+	    {
+		/* end of line or next character not a letter */
+		if (((current_x + strlen(prevanswer)) == strlen(fileptr->data))
+			|| (!isalpha((int) fileptr->data[current_x + strlen(prevanswer)])))
+		    ;
+		else
+		    continue;
+	    }
+	    else
+		continue;
+	}
+
 	/* If we're here, we've found the search string */
 	if (!replaceall)
-	    i = do_yesno(1, 1, _("Replace this instance?"));
+	    *i = do_yesno(1, 1, _("Replace this instance?"));
 
-	if (i > 0 || replaceall) {	/* Yes, replace it!!!! */
-	    if (i == 2)
+	if (*i > 0 || replaceall) {	/* Yes, replace it!!!! */
+	    if (*i == 2)
 		replaceall = 1;
 
-            copy = replace_line();
-            if (!copy) {
-                statusbar("Replace failed: unknown subexpression!");
-                replace_abort();
+	    copy = replace_line();
+	    if (!copy) {
+		statusbar(_("Replace failed: unknown subexpression!"));
+		replace_abort();
 		return 0;
-            }
+	    }
 
 	    /* Cleanup */
 	    free(current->data);
@@ -479,15 +539,98 @@ int do_replace(void)
 	    /* Stop bug where we replace a substring of the replacement text */
 	    current_x += strlen(last_replace) - 1;
 
+	    /* Adjust the original cursor position - COULD BE IMPROVED */
+	    if (search_last_line) {
+		*beginx += strlen(last_replace) - strlen(last_search);
+
+		/* For strings that cross the search start/end boundary */
+		/* Don't go outside of allocated memory */
+		if (*beginx < 1)
+		    *beginx = 1;
+	    }
+
 	    edit_refresh();
 	    set_modified();
 	    numreplaced++;
-	} else if (i == -1)	/* Abort, else do nothing and continue loop */
+	} else if (*i == -1)	/* Abort, else do nothing and continue loop */
 	    break;
     }
 
+    return numreplaced;
+}
+
+/* Replace a string */
+int do_replace(void)
+{
+    int i, numreplaced, beginx;
+    filestruct *begin;
+    char *prevanswer = NULL, *buf = NULL;
+
+    i = search_init(1);
+    switch (i) {
+    case -1:
+	statusbar(_("Replace Cancelled"));
+	replace_abort();
+	return 0;
+    case 1:
+	do_replace();
+	return 1;
+    case -2:
+	do_search();
+	return 0;
+    case -3:
+	replace_abort();
+	return 0;
+    }
+
+    /* Again, there was a previous string but they deleted it and hit enter */
+    if (!ISSET(PICO_MODE) && !strcmp(answer, "")) {
+	statusbar(_("Replace Cancelled"));
+	replace_abort();
+	return 0;
+    }
+
+     /* If answer is now == "", then PICO_MODE is set.  So, copy
+	last_search into answer (and prevanswer)... */
+    if (!strcmp(answer, "")) {
+	answer = mallocstrcpy(answer, last_search);
+	prevanswer = mallocstrcpy(prevanswer, last_search);
+    } else {
+	last_search = mallocstrcpy(last_search, answer);
+	prevanswer = mallocstrcpy(prevanswer, answer);
+    }
+
+    if (ISSET(PICO_MODE)) {
+	buf = nmalloc(strlen(last_replace) + 5);
+	if (strcmp(last_replace, "")) {
+	    if (strlen(last_replace) > (COLS / 3)) {
+		strncpy(buf, last_replace, COLS / 3);
+		sprintf(&buf[COLS / 3 - 1], "...");
+	    } else
+		sprintf(buf, "%s", last_replace);
+
+	    i = statusq(0, replace_list_2, REPLACE_LIST_2_LEN, "",
+			_("Replace with [%s]"), buf);
+	}
+	else
+	    i = statusq(0, replace_list_2, REPLACE_LIST_2_LEN, "",
+			_("Replace with"));
+    }
+    else
+	i = statusq(0, replace_list_2, REPLACE_LIST_2_LEN, last_replace, 
+			_("Replace with"));
+
+    /* save where we are */
+    begin = current;
+    beginx = current_x + 1;
+
+    search_last_line = 0;
+
+    numreplaced = do_replace_loop(prevanswer, begin, &beginx, FALSE, &i);
+
+    /* restore where we were */
     current = begin;
-    current_x = beginx;
+    current_x = beginx - 1;
     renumber_all();
     edit_update(current, CENTER);
     print_replaced(numreplaced);
@@ -510,7 +653,7 @@ int do_gotoline(long defline)
 	line = defline;
     else {			/* Ask for it */
 
-	j = statusq(goto_list, GOTO_LIST_LEN, "", _("Enter line number"));
+	j = statusq(0, goto_list, GOTO_LIST_LEN, "", _("Enter line number"));
 	if (j == -1) {
 	    statusbar(_("Aborted"));
 	    goto_abort();
